/***************************************************************************
 *   Authors:                                                              *
 *   - Nicolas Brodu                                                       *
 *                                                                         *
 *   Copyrights:                                                           *
 *   - INRIA                                                               *
 *                                                                         *
 *   This file is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as               *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include "ClassifierInterface.h"

#include "Utility.h"

ClassifierInterface::ClassifierInterface() : quiet(false), verbose(false) {
}

ClassifierInterface::~ClassifierInterface() {
}

std::string ClassifierInterface::getName() {
    return "Unknown classifier";
}

bool ClassifierInterface::setVFoldCrossValidation(int v) {
    return false;
}

bool ClassifierInterface::supportWeightedInstance() {
    return false;
}

void ClassifierInterface::prepareForTraining(int ninstances, int instancedim) {
    trainSet.clear();
    trainClasses.clear();
    trainWeights.clear();
    cumulatedWeights.clear();
    trainSet.reserve(ninstances);
    trainClasses.reserve(ninstances);
    trainWeights.reserve(ninstances);
    cumulatedWeights.reserve(ninstances);
}

void ClassifierInterface::train(const itpp::vec& instance, double classification, double instanceWeight) {
    trainSet.push_back(instance);
    trainClasses.push_back(classification);
    trainWeights.push_back(instanceWeight);
    if (cumulatedWeights.empty()) cumulatedWeights.push_back(instanceWeight);
    else cumulatedWeights.push_back(instanceWeight + cumulatedWeights.back());
}

double ClassifierInterface::finishTraining() {
    return 1.0;
}

bool ClassifierInterface::parseOption(int option, std::string option_arg) {
    return false;
}

bool ClassifierInterface::getQuiet() {
    return quiet;
}

void ClassifierInterface::setQuiet(bool q) {
    quiet = q;
}

bool ClassifierInterface::getVerbose() {
    return verbose;
}

void ClassifierInterface::setVerbose(bool v) {
    verbose = v;
}


ClassifierFactory::ClassifierFactory() {
}

ClassifierFactory::~ClassifierFactory() {
}

void ClassifierFactory::printClassifierHelpText(std::ostream& os) {
}

std::string ClassifierFactory::getClassifierOptions() {
    return "";
}

ClassifierFactory::ClassifierInputType ClassifierFactory::getType() {
    return ClassifierFactory::FeatureVector;
}

int ClassifierInterface::drawRandomInstance() {
    // the selected instance is the one for which cumw(instance-1) < cw <= cumw(instance) with cumw(-1) = 0;
    double cw = Utility::Random() * cumulatedWeights.back();
    // dicho search, cumulatedWeights is sorted as weigths are >=0.
    int i0 = 0;
    int i1 = cumulatedWeights.size() - 1;
    while (i1>i0) {
        int imid = (i0 + i1) / 2;
        double wmid = cumulatedWeights[imid];
        if (cw > wmid) i0 = imid+1;
        else i1 = imid;
    }
    return i0;
}





