/***************************************************************************
 *   Authors:                                                              *
 *   - Nicolas Brodu                                                       *
 *                                                                         *
 *   Copyrights:                                                           *
 *   - INRIA                                                               *
 *   - Nicolas Brodu (initial base code, some modifications in SVN)        *
 *                                                                         *
 *   This file is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as               *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef CLASSIFIER_LinearFisherDiscriminant
#define CLASSIFIER_LinearFisherDiscriminant

#include <vector>

#include "../ClassifierInterface.h"

struct LinearFisherDiscriminantClassifier : public ClassifierInterface {
    //false: TODO: implement weighted gaussian classes
    //bool supportWeightedInstance();
    void prepareForTraining(int ninstances, int instancedim);
    void train(const itpp::vec& instance, double classification, double instanceWeight = 1.0);
    double finishTraining();
    double predict(const itpp::vec& instance);
    std::string getName();

protected:
    std::vector<itpp::vec> mean;
    std::vector<itpp::mat> cov;
    itpp::vec gw;
    int ninstances;
    itpp::vec discriminantsW;
    double discriminantsT;
};

struct LinearFisherDiscriminantFactory : public ClassifierFactory {
    ClassifierInputType getType();
    ClassifierInterface* instanciateClassifier();
    void printClassifierHelpText(std::ostream& os);
    int getMethodId();
};

#endif
