/***************************************************************************
 *   Authors:                                                              *
 *   - Nicolas Brodu                                                       *
 *                                                                         *
 *   Copyrights:                                                           *
 *   - INRIA                                                               *
 *   - Nicolas Brodu (initial base code, some modifications in SVN)        *
 *                                                                         *
 *   This file is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as               *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

// ITPP v>=4 only
#ifndef USE_ITPP_V3_10
#include <itpp/base/math/log_exp.h>
#else
#include <itpp/base/scalfunc.h>
#endif

#include "MultivariateStudentT.h"

using namespace std;
using namespace itpp;

void MultivariateStudentT::prepareForEstimation() {
    int d = mean.size();
    powConstant = 0.5*(d + dof);
    double lg1 = lgamma(powConstant); // applied on positive args, so >0, no need for checking signs
    double lg2 = lgamma(0.5*dof);
    double logConst1 = 0.5 * d * log(dof * 3.14159265358979323846);
    double logConst2 = 0.5 * log(fabs(det(precision)));
    factorConstant = lg1 - lg2 - logConst1 + logConst2;
}

double MultivariateStudentT::logDensity(const itpp::vec& x) {
    vec xdelta = x - mean;
    mat tmp = xdelta.transpose() * precision * xdelta;
    return factorConstant - powConstant * log(1.0 + tmp(0,0) / dof);
}

double MultivariateStudentT::density(const itpp::vec& x) {
    return trunc_exp(logDensity(x));
}
