/***************************************************************************
 *   Authors:                                                              *
 *   - Nicolas Brodu                                                       *
 *   - Makoto Matsumoto and Takuji Nishimura (see below)                   *
 *   - Mutsuo Saito (see below)                                            *
 *   - Phil Burk (see below)                                               *
 *                                                                         *
 *   Copyrights:                                                           *
 *   - Nicolas Brodu (initial author of incremfa)                          *
 *   - Makoto Matsumoto and Takuji Nishimura (see below)                   *
 *   - Mutsuo Saito (see below)                                            *
 *   - Phil Burk (see below)                                               *
 *   - INRIA (modifications traced in SVN)                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Utility.h"
#include <iostream>
using namespace std;
#include <time.h>

#include <math.h>

using namespace Utility;


//////////////////////////////////////////////////////////////////////
// Code stolen and adapted from mt19937ar.c
//////////////////////////////////////////////////////////////////////

/*
   A C-program for MT19937, with initialization improved 2002/1/26.
   Coded by Takuji Nishimura and Makoto Matsumoto.

   Before using, initialize the state by using init_genrand(seed)  
   or init_by_array(init_key, key_length).

   Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
   All rights reserved.                          
   Copyright (C) 2005, Mutsuo Saito,
   All rights reserved.                          

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote 
        products derived from this software without specific prior written 
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


   Any feedback is very welcome.
   http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
   email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)
*/


/* Period parameters */
#define N 624
#define M 397
#define MATRIX_A 0x9908b0dfUL   /* constant vector a */
#define UPPER_MASK 0x80000000UL /* most significant w-r bits */
#define LOWER_MASK 0x7fffffffUL /* least significant r bits */

/* initializes mt[N] with a seed */
inline void init_genrand(unsigned int s, RandomState& state)
{
    state.seed = s;
    state.mt[0]= s; // & 0xffffffffUL; // NB060508: unnecessary with the use of sized types
    for (state.mti=1; state.mti<N; state.mti++) {
        state.mt[state.mti] = 
        (1812433253UL * (state.mt[state.mti-1] ^ (state.mt[state.mti-1] >> 30)) + state.mti);
        /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
        /* In the previous versions, MSBs of the seed affect   */
        /* only MSBs of the array mt[].                        */
        /* 2002/01/09 modified by Makoto Matsumoto             */
        //mt[mti] &= 0xffffffffUL;  // NB060508: unnecessary with the use of sized types
        /* for >32 bit machines */
    }
}

/* generates a random number on [0,0xffffffff]-interval */
inline unsigned int genrand_int(RandomState& state)
{
    unsigned int y;
    static unsigned int mag01[2]={0x0UL, MATRIX_A};
    /* mag01[x] = x * MATRIX_A  for x=0,1 */

    if (state.mti >= N) { /* generate N words at one time */
        int kk;

        //if (state.mti == N+1)   /* if init_genrand() has not been called, */
            //init_genrand(5489UL, state); /* a default initial seed is used */

        for (kk=0;kk<N-M;kk++) {
            y = (state.mt[kk]&UPPER_MASK)|(state.mt[kk+1]&LOWER_MASK);
            state.mt[kk] = state.mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1UL];
        }
        for (;kk<N-1;kk++) {
            y = (state.mt[kk]&UPPER_MASK)|(state.mt[kk+1]&LOWER_MASK);
            state.mt[kk] = state.mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1UL];
        }
        y = (state.mt[N-1]&UPPER_MASK)|(state.mt[0]&LOWER_MASK);
        state.mt[N-1] = state.mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1UL];

        state.mti = 0;
    }
  
    y = state.mt[state.mti++];

    /* Tempering */
    y ^= (y >> 11);
    y ^= (y << 7) & 0x9d2c5680UL;
    y ^= (y << 15) & 0xefc60000UL;
    y ^= (y >> 18);

    return y;
}


//////////////////////////////////////////////////////////////////////
// End of code adapted from mt19937ar.c
//////////////////////////////////////////////////////////////////////

namespace Utility {

// Default state holder, so single threaded applications don't bother setting up an object
RandomState DefaultRandomState;

}

/*
    patest_pink.c

    Generate Pink Noise using Gardner method.
    Optimization suggested by James McCartney uses a tree
    to select which random value to replace.

    x x x x x x x x x x x x x x x x 
     x   x   x   x   x   x   x   x   
       x       x       x       x       
           x               x               
                   x                               

    Tree is generated by counting trailing zeros in an increasing index.
    When the index is zero, no random number is selected.

    Author: Phil Burk, http://www.softsynth.com
    
    Revision History:

    Copyleft 1999 Phil Burk - No rights reserved.
*/


#define PINK_MAX_RANDOM_ROWS   (30)
#define PINK_RANDOM_BITS       (24)
#define PINK_RANDOM_SHIFT      ((sizeof(long)*8)-PINK_RANDOM_BITS)

typedef struct
{
    long      pink_Rows[PINK_MAX_RANDOM_ROWS];
    long      pink_RunningSum;   /* Used to optimize summing of generators. */
    int       pink_Index;        /* Incremented each sample. */
    int       pink_IndexMask;    /* Index wrapped by ANDing with this mask. */
    double    pink_Scalar;       /* Used to scale within range of -1.0 to +1.0 */
} PinkNoise;

/* Setup PinkNoise structure for N rows of generators. */
void InitializePinkNoise( PinkNoise *pink, int numRows )
{
    int i;
    long pmax;
    pink->pink_Index = 0;
    pink->pink_IndexMask = (1<<numRows) - 1;
/* Calculate maximum possible signed random value. Extra 1 for white noise always added. */
    pmax = (numRows + 1) * (1<<(PINK_RANDOM_BITS-1));
    pink->pink_Scalar = 1.0f / pmax;
/* Initialize rows. */
    for( i=0; i<numRows; i++ ) pink->pink_Rows[i] = 0;
    pink->pink_RunningSum = 0;
}

/* Generate Pink noise values between -1.0 and +1.0 */
double GeneratePinkNoise( PinkNoise *pink, RandomState& state = DefaultRandomState )
{
    long newRandom;
    long sum;
    double output;

/* Increment and mask index. */
    pink->pink_Index = (pink->pink_Index + 1) & pink->pink_IndexMask;

/* If index is zero, don't update any random values. */
    if( pink->pink_Index != 0 )
    {
    /* Determine how many trailing zeros in PinkIndex. */
    /* This algorithm will hang if n==0 so test first. */
        int numZeros = 0;
        int n = pink->pink_Index;
        while( (n & 1) == 0 )
        {
            n = n >> 1;
            numZeros++;
        }

    /* Replace the indexed ROWS random value.
     * Subtract and add back to RunningSum instead of adding all the random
     * values together. Only one changes each time.
     */
        pink->pink_RunningSum -= pink->pink_Rows[numZeros];
        newRandom = ((long)genrand_int(state)) >> PINK_RANDOM_SHIFT;
        pink->pink_RunningSum += newRandom;
        pink->pink_Rows[numZeros] = newRandom;
    }
    
/* Add extra white noise value. */
    newRandom = ((long)genrand_int(state)) >> PINK_RANDOM_SHIFT;
    sum = pink->pink_RunningSum + newRandom;

/* Scale to range of -1.0 to 0.9999. */
    output = pink->pink_Scalar * sum;

    return output;
}


namespace Utility {

// Uninitialized pink noise
static PinkNoise thePinkNoise;

long RandomInit(RandomState& state) {
    RandomInit(time(0), state);
    return state.seed;
}

void RandomInit(long seed, RandomState& state) {
    state.seed = seed;
    init_genrand(seed,state);
    PinkNoiseSetRows();
    cerr << "Random seed=" << state.seed << endl;
}

long RandomSeed(RandomState& state) {
    return state.seed;
}

void PinkNoiseSetRows(int nrows) {
    InitializePinkNoise(&thePinkNoise, nrows);
}

// From Streflop-0.3 (Nicolas Brodu)
static inline unsigned int getRestrictedRandomInt(unsigned int n, RandomState& state)
{
    // First propagate leading 1 to all other bits
    unsigned int mask = n;
    mask |= mask >> 1;
    mask |= mask >> 2;
    mask |= mask >> 4;
    mask |= mask >> 8;
    mask |= mask >> 16;

    // Draw only that number of bits, until a number is in the desired range [0,n]
    // Worse case is number of loops proba decreasing in 1/2^nloops
    unsigned int ret;
    do {
        ret = ((unsigned int)genrand_int(state)) & mask;
    } while( ret > n );

    return ret;
}

int IRandom(int min, int max, RandomState& state) {
    if (max < min) return IRandom(max, min, state);
    return getRestrictedRandomInt(max-min,state)+min;
}


double Random(RandomState& state) {
    // Generate bits
    unsigned int ret = genrand_int(state);

    unsigned int maxInt = (unsigned int)(-1);

    return (double)ret / ((double)maxInt + 1.0);
}

unsigned long BRandom(RandomState& state) {
    return (unsigned long)genrand_int(state);
}

#define TWO_PI (2.0*3.1415926535897932384626433832795029)

double NRandom(RandomState& state) {
    double x;
    do {
        x = Random(state);
    } while (x==0.0);
    x = sqrt(-2.0 * log(x));
    if (Random(state) > 0.5) return x * cos(TWO_PI * Random(state));
    return x * sin(TWO_PI * Random(state));
}

double PRandom(RandomState& state) {
    return (GeneratePinkNoise(&thePinkNoise, state)+1.0)*0.5;
}

double b0=0.0, b1=0.0, b2=0.0, b3=0.0, b4=0.0, b5=0.0, b6=0.0;
double PRandom2(RandomState& state) {
    double white = Random(state);
    b0 = 0.99886 * b0 + white * 0.0555179;
    b1 = 0.99332 * b1 + white * 0.0750759;
    b2 = 0.96900 * b2 + white * 0.1538520;
    b3 = 0.86650 * b3 + white * 0.3104856;
    b4 = 0.55000 * b4 + white * 0.5329522;
    b5 = -0.7616 * b5 - white * 0.0168980;
    double pink = b0 + b1 + b2 + b3 + b4 + b5 + b6 + white * 0.5362;
    b6 = white * 0.115926;
    return pink;
}

} // end namespace

