/*
 * This file is adapted from the public-domain lmfit project on sourceforge
 * It regroups all original useful files so as to eash compilation/calling from the main code.
 *
 * See also:
 * - Netlib MINPACK original fortran code: http://www.netlib.org/minpack/
 * - The lmfit C implementation: https://sourceforge.net/projects/lmfit/
 *
 * lmfit
 *
 * Solves or minimizes the sum of squares of m nonlinear
 * functions of n variables.
 *
 * From public domain Fortran version
 * of Argonne National Laboratories MINPACK
 *     argonne national laboratory. minpack project. march 1980.
 *     burton s. garbow, kenneth e. hillstrom, jorge j. more
 * C translation by Steve Moshier
 * Joachim Wuttke converted the source into C++ compatible ANSI style
 * and provided a simplified interface
 * Nicolas Brodu removed the simplified interface for the incremfa project
 * since in the present case it actually complexifies the code by adding an extra layer
 */

#ifndef LMFIT_H
#define LMFIT_H

// the subroutine that calculates fvec:
typedef void (lm_evaluate_ftype) (double *par, int m_dat, double *fvec,
				  void *data, int *info);

// the subroutine that informs about fit progress:
typedef void (lm_print_ftype) (int n_par, double *par, int m_dat,
			       double *fvec, void *data, int iflag,
			       int iter, int nfev);

double lm_enorm(int, double *);

// low-level interface for full control:
void lm_lmdif(int m, int n, double *x, double *fvec, double ftol,
	      double xtol, double gtol, int maxfev, double epsfcn,
	      double *diag, int mode, double factor, int *info, int *nfev,
	      double *fjac, int *ipvt, double *qtf, double *wa1,
	      double *wa2, double *wa3, double *wa4,
	      lm_evaluate_ftype * evaluate, lm_print_ftype * printout,
	      void *data);

#endif
